<?php

/**
 * @author Daniel Dimitrov <daniel@compojoom.com>
 * @copyright    Copyright (C) 2008 - 2023 compojoom.com. All rights reserved.
 * @license        GNU General Public License version 2 or later
 */

// no direct access
defined('_JEXEC') or die;


use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Updater\Updater;
use Joomla\CMS\Mail\Mail;

/**
 * Joomla! Update notification plugin
 * This plugin checks at specific intervals for new updates
 * and sends a notification if an update is found
 *
 * Special thanks to O.Schwab <service@castle4us.de> for coming with the idea
 * in the first place :)
 *
 * @package        Joomla.Plugin
 * @subpackage    System.cupdater
 */
class plgSystemCupdater extends CMSPlugin

{
    private $recipients = array();

    /**
     * @return bool
     */
    public function onAfterRender()
    {
        if (!$this->doIHaveToRun()) {
            return false;
        }

//      so we are running??? Then let us load some languages
        $lang = Factory::getLanguage();
        $lang->load('plg_system_cupdater', JPATH_ADMINISTRATOR, 'en-GB', true);
        $lang->load('plg_system_cupdater', JPATH_ADMINISTRATOR, $lang->getDefault(), true);
        $lang->load('plg_system_cupdater', JPATH_ADMINISTRATOR, null, true);


        // clear the cache - otherwise we get problems in backend
        $cache = Factory::getCache();
        $cache->clean('com_plugins');

		// note for Joomla 3
		// the JHttpTransportCurl class is throwing an exception when it is unable to get the content out
		// of an url. The exception is not caught in the Joomla updater class and this leads to a nasty
		// exception thrown out at the user. That is why we catch it here and send an email to the
		// people that should be notified about that
		// normally joomla should disable such urls, but since they don't react on the Exception this will
		// happen each time the update script is run
		// that is why the administrator will need to find the url causing the problem and disable it for the server
		try {
			$updates = $this->checkForUpdates();
		} catch (UnexpectedValueException $e) {
			// let us send a mail to the user
			$title = Text::_('PLG_CUPDATER_ERROR_UPDATE');
			$body = Text::_('PLG_CUPDATER_ERROR_UPDATE_DESC');
			$body .= "\n" . Text::_('PLG_CUPDATER_DISCLAIMER') . "\n";
			$this->sendMail($title, $body);
			$this->setLastRunTimestamp();
			// and exit...
			return false;
		}

        if (count($updates)) {
            if (count($updates) == 1) {
                $title = Text::_('PLG_CUPDATER_FOUND_ONE_UPDATE');
            } else {
                $title = Text::sprintf('PLG_CUPDATER_FOUND_UPDATES', count($updates));
            }

            $body = '';
            foreach ($updates as $value) {
                $body .= Text::_('PLG_CUPDATER_UPDATE_FOR') . ': '
                    . $value->name . ' ' . Text::_('PLG_CUPDATER_VERSION') . ': '
                    . $value->version . ' ' . Text::_('PLG_CUPDATER_FOUND') . "\n";
            }
            $body .= "\n" . Text::_('PLG_CUPDATER_TO_APPLY_UPDATE') . ': '
                . Uri::base() . ' ' . Text::_('PLG_CUPDATER_AND_GO_TO') . "\n";

        } else if ((!count($updates)) && (($this->params->get('mailto_noresult') == 0))) {
            $title = Text::_('PLG_CUPDATER_FOUND_NO_UPDATE_TITLE');
            $body = Text::_('PLG_CUPDATER_FOUND_NO_UPDATE') . ' ' . Uri::base() . "\n";
        }

        if (count($updates) ||
            ((!count($updates)) && (($this->params->get('mailto_noresult') == 0)))) {

            $body .= $this->nextUpdateText();
            $body .= "\n" . Text::_('PLG_CUPDATER_DISCLAIMER') . "\n";

            $this->sendMail($title, $body);
        }

        $this->setLastRunTimestamp();

        return true;
    }

    /**
     * @return string - our next update text
     */
    private function nextUpdateText()
    {
        $body = Text::_("PLG_CUPDATER_NEXT_UPDATE_CHECK_WILL_BE") . ' ';

        switch ($this->params->get('notification_period')) {
            case "24":
                $body .= Text::_('PLG_CUPDATER_TOMORROW') . '.';
                break;
            case "168":
                $body .= Text::_('PLG_CUPDATER_IN_ONE_WEEK') . '.';
                break;
            case "336":
                $body .= Text::sprintf('PLG_CUPDATER_IN_WEEKS', 2) . '.';
                break;
            case "672":
                $body .= Text::sprintf('PLG_CUPDATER_WEEKS', 4) . '.';
                break;
        }

        return $body;
    }

    /**
     * @param $title - the title of the mail
     * @param $body - the body of the mail
     */
	private function sendMail($title, $body)
	{
		$app = Factory::getApplication();
		$recipients = $this->getRecipients();
		if (!count($recipients)) {
			return;
		}
		$mail = Mail::getInstance();
		$mail->addRecipient($recipients);
		$mail->setSender(array($app->get('mailfrom'), $app->get('fromname')));
		$mail->setSubject($title);
		$mail->setBody($body);

		$mail->Send();
	}

    /**
     *
     * @return array - all recepients
     */
    private function getRecipients()
    {
        $emails = array();

        if (!count($this->recipients)) {
            if ($this->params->get('mailto_admins', 0)) {
                $groups = $this->params->get('mailto_admins');
                $tmp_emails = $this->getEmailsForUsersInGroups($groups);

                if (!is_array($emails)) {
                    $emails = array();
                }
                $emails = array_merge($tmp_emails, $emails);
            }

            if ((int)$this->params->get('mailto_custom') == 1 && $this->params->get('custom_email') != "") {
                $tmp_emails = explode(';', $this->params->get('custom_email'));

                if (!is_array($emails)) {
                    $emails = array();
                }
                $emails = array_merge($tmp_emails, $emails);
            }

            $tmp = array();
            foreach ($emails AS $r)
            {
                if (in_array($r, $tmp) || trim($r) == "") {
                    continue;
                }
                else {
                    $this->recipients[] = $r;
                    $tmp[] = $r;
                }
            }
        }

        return $this->recipients;
    }

    /**
     * checks if there are updates available and
     * @return object - the updates
     */
	private function checkForUpdates()
	{
		// Use Joomla 4's namespaced class for Updater
		$updater = Updater::getInstance();
		$updater->findUpdates(0, 0);

		// Use Factory to get the database object
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->select('*');
		$query->from($db->quoteName('#__updates'))->where($db->quoteName('extension_id') . ' != 0');
		$db->setQuery($query);

		return $db->loadObjectList();
	}

    /**
     * "Do I have to run?" - the age old question. Let it be answered by checking the
     * last execution timestamp, stored in the component's configuration.
     *
     * this function is copied from the asexpirationnotify plugin so all credits go to
     * Nicholas K. Dionysopoulos / AkeebaBackup.com
     * @return bool
     */
    private function doIHaveToRun()
    {
        $params = $this->params;
        $lastRunUnix = $params->get('plg_cupdate_timestamp', 0);
        $dateInfo = getdate($lastRunUnix);
        $nextRunUnix = mktime(0, 0, 0, $dateInfo['mon'], $dateInfo['mday'], $dateInfo['year']);
        $nextRunUnix += $params->get('notification_period', 24) * 3600;
        $now = time();

        return ($now >= $nextRunUnix);
    }

    /**
     * Saves the timestamp of this plugin's last run
     * this function is copied from the asexpirationnotify plugin so all credits go to
     * Nicholas K. Dionysopoulos / AkeebaBackup.com
     *
     */
    private function setLastRunTimestamp()
    {
	    $lastRun = time();
	    $params = $this->params;
	    $params->set('plg_cupdate_timestamp', $lastRun);

	    $db = Factory::getDBO();

	    $data = $params->toString('JSON');
	    $query = $db->getQuery(true);
	    $query->update($db->quoteName('#__extensions'));
	    $query->set($db->quoteName('params') . ' = ' . $db->quote($data));
	    $query->where($db->quoteName('element') . ' = ' . $db->quote('cupdater') . ' AND ' . $db->quoteName('type') . ' = ' . $db->quote('plugin'));
	    $db->setQuery($query);
	    $db->execute();
    }

    /**
     *
     * @param array $groups - the user group
     * @return mixed
     */
    private function getEmailsForUsersInGroups(array $groups)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('a.email');
        $query->from('#__user_usergroup_map AS map');
        $query->leftJoin('#__users AS a ON a.id = map.user_id');
        $query->where('map.group_id IN (' . implode(',', $groups) . ')');

        $db->setQuery($query);
        return $db->loadColumn();
    }

}
